@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1996, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2020, MariaDB Corporation.
+Copyright (c) 2017, 2021, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -513,7 +513,9 @@ trx_sys_init_at_db_start()
 		     trx = UT_LIST_GET_NEXT(trx_list, trx)) {
 
 			ut_ad(trx->is_recovered);
-			assert_trx_in_rw_list(trx);
+			ut_ad(!trx->read_only);
+			ut_ad(trx->in_rw_trx_list);
+			ut_ad(!trx->is_autocommit_non_locking());
 
 			if (trx_state_eq(trx, TRX_STATE_ACTIVE)) {
 				rows_to_undo += trx->undo_no;
@@ -997,11 +999,6 @@ trx_sys_any_active_transactions(void)
 		/* This may count some ACTIVE transactions twice,
 		both in rw_trx_list and mysql_trx_list. */
 		total_trx += trx->state == TRX_STATE_ACTIVE;
-		/* Any PREPARED or COMMITTED transactions must be
-		in rw_trx_list, so it suffices to count them there. */
-		ut_ad(trx->in_rw_trx_list
-		      || trx->state == TRX_STATE_NOT_STARTED
-		      || trx->state == TRX_STATE_ACTIVE);
 		trx_mutex_exit(trx);
 	}
 
@@ -1031,7 +1028,8 @@ trx_sys_validate_trx_list_low(
 	     trx != NULL;
 	     prev_trx = trx, trx = UT_LIST_GET_NEXT(trx_list, prev_trx)) {
 
-		check_trx_state(trx);
+		ut_ad(!trx->is_autocommit_non_locking());
+		ut_ad(trx->state != TRX_STATE_NOT_STARTED);
 		ut_a(prev_trx == NULL || prev_trx->id > trx->id);
 	}
 
